@@ -321,22 +321,9 @@ static gboolean
 tick_timeout (RBPlayerGst *mp)
 {
 	if (mp->priv->playing) {
-		gint64 position;
-
-		position = rb_player_get_time (RB_PLAYER (mp));
-
-		/* if we don't have stream-changed messages, do the track change when
-		 * the playback position is less than one second into the current track,
-		 * which pretty much has to be the new one.
-		 */
-		if (mp->priv->playbin_stream_changing && (position < GST_SECOND)) {
-			emit_playing_stream_and_tags (mp, TRUE);
-			mp->priv->playbin_stream_changing = FALSE;
-		}
-
 		_rb_player_emit_tick (RB_PLAYER (mp),
 				      mp->priv->stream_data,
-				      position,
+				      rb_player_get_time (RB_PLAYER (mp)),
 				      -1);
 	}
 	return TRUE;
@@ -441,6 +428,16 @@ state_change_finished (RBPlayerGst *mp, GError *error)
 		if (error != NULL) {
 			g_warning ("unable to stop playback: %s\n", error->message);
 		} else {
+			GstBus *bus;
+
+			/* flush bus to ensure tags from the previous stream don't
+			 * get applied to the new one
+			 */
+			bus = gst_element_get_bus (mp->priv->playbin);
+			gst_bus_set_flushing (bus, TRUE);
+			gst_bus_set_flushing (bus, FALSE);
+			gst_object_unref (bus);
+
 			rb_debug ("setting new playback URI %s", mp->priv->uri);
 			g_object_set (mp->priv->playbin, "uri", mp->priv->uri, NULL);
 			start_state_change (mp, GST_STATE_PLAYING, FINISH_TRACK_CHANGE);
@@ -577,9 +574,15 @@ bus_cb (GstBus *bus, GstMessage *message, RBPlayerGst *mp)
 
 	case GST_MESSAGE_TAG: {
 		GstTagList *tags;
+
+		if (mp->priv->playbin_stream_changing) {
+			rb_debug ("ignoring tags during playbin stream change");
+			break;
+		}
+
 		gst_message_parse_tag (message, &tags);
 
-		if (mp->priv->stream_change_pending || mp->priv->playbin_stream_changing) {
+		if (mp->priv->stream_change_pending) {
 			mp->priv->stream_tags = g_list_append (mp->priv->stream_tags, tags);
 		} else {
 			gst_tag_list_foreach (tags, (GstTagForeachFunc) process_tag, mp);
